@@ -21,27 +21,33 @@ package org.jcodings.transcode;
 
 import static org.jcodings.util.CaseInsensitiveBytesHash.caseInsensitiveEquals;
 
+import org.jcodings.Encoding;
 import org.jcodings.Ptr;
 import org.jcodings.exception.InternalException;
+import org.jcodings.specific.UTF32BEEncoding;
+
+import java.util.Arrays;
 
 public final class EConv implements EConvFlags {
     int flags;
-    byte[] source, destination; // source/destination encoding names
+    public byte[] source, destination; // source/destination encoding names
 
     boolean started = false;
 
-    byte[] replacementString;
-    int replacementLength;
-    byte[] replacementEncoding;
+    public byte[] replacementString;
+    public int replacementLength;
+    public byte[] replacementEncoding;
 
     Buffer inBuf = new Buffer();
 
-    EConvElement[] elements;
-    int numTranscoders;
+    public EConvElement[] elements;
+    public int numTranscoders;
     int numFinished;
 
-    Transcoding lastTranscoding;
-    final LastError lastError = new LastError();
+    public Transcoding lastTranscoding;
+    public final LastError lastError = new LastError();
+
+    public Encoding sourceEncoding, destinationEncoding;
 
     @Override
     public String toString() {
@@ -49,18 +55,18 @@ public final class EConv implements EConvFlags {
     }
 
     EConv(int nHint) {
-        if (nHint < 0) nHint = 1;
+        if (nHint <= 0) nHint = 1;
         elements = new EConvElement[nHint];
         lastError.result = EConvResult.SourceBufferEmpty;
     }
 
-    static final class EConvElement extends Buffer {
+    public static final class EConvElement extends Buffer {
         EConvElement(Transcoding transcoding) {
             this.transcoding = transcoding;
             lastResult = EConvResult.SourceBufferEmpty;
         }
 
-        final Transcoding transcoding;
+        public final Transcoding transcoding;
         EConvResult lastResult;
 
         @Override
@@ -71,13 +77,13 @@ public final class EConv implements EConvFlags {
         }
     }
 
-    static final class LastError {
+    public static final class LastError {
         EConvResult result;
         Transcoding errorTranscoding;
         byte[] source, destination;
 
         byte[] errorBytes;
-        int errorBytesP, errorBytesEnd;
+        int errorBytesP, errorBytesLength;
         int readAgainLength;
 
         void reset() {
@@ -85,16 +91,48 @@ public final class EConv implements EConvFlags {
             errorTranscoding = null;
             source = destination = null;
             errorBytes = null;
-            errorBytesP = errorBytesEnd = 0;
+            errorBytesP = errorBytesLength = 0;
             readAgainLength = 0;
         }
 
+        public EConvResult getResult() {
+            return result;
+        }
+
+        public Transcoding getErrorTranscoding() {
+            return errorTranscoding;
+        }
+
+        public byte[] getSource() {
+            return source;
+        }
+
+        public byte[] getDestination() {
+            return destination;
+        }
+
+        public byte[] getErrorBytes() {
+            return errorBytes;
+        }
+
+        public int getErrorBytesP() {
+            return errorBytesP;
+        }
+
+        public int getErrorBytesLength() {
+            return errorBytesLength;
+        }
+
+        public int getReadAgainLength() {
+            return readAgainLength;
+        }
+
         @Override
         public String toString() {
             String s = "Last Error " + (source == null ? "null" : new String(source)) + " => " + (destination == null ? "null" : new String(destination))
                     + "\n";
             s += "  result: " + result.toString() + "\n";
-            s += "  error bytes: " + (errorBytes == null ? "null" : new String(errorBytes, errorBytesP, errorBytesP + errorBytesEnd)) + "\n";
+            s += "  error bytes: " + (errorBytes == null ? "null" : new String(errorBytes, errorBytesP, errorBytesP + errorBytesLength)) + "\n";
             s += "  read again length: " + readAgainLength;
             return s;
         }
@@ -138,28 +176,34 @@ public final class EConv implements EConvFlags {
     private int transSweep(byte[] in, Ptr inPtr, int inStop, byte[] out, Ptr outPtr, int outStop, int flags, int start) {
         boolean try_ = true;
 
-        final Ptr ipp = new Ptr();
-        final Ptr opp = new Ptr();
+        Ptr ipp = null;
+        Ptr opp = null;
 
         while (try_) {
-            for (int i = 0; i < numTranscoders; i++) {
+            try_ = false;
+            for (int i = start; i < numTranscoders; i++) {
                 EConvElement te = elements[i];
 
                 final int is, os;
                 final byte[] ibytes, obytes;
+                EConvElement previousTE = null;
+                boolean ippIsStart = false;
+                boolean oppIsEnd = false;
+
                 if (i == 0) {
-                    ipp.p = inPtr.p;
+                    ipp = inPtr;
                     is = inStop;
                     ibytes = in;
                 } else {
-                    EConvElement previousTE = elements[i - 1];
-                    ipp.p = previousTE.dataStart; // !!!!
+                    previousTE = elements[i - 1];
+                    ipp = new Ptr(previousTE.dataStart);
+                    ippIsStart = true;
                     is = previousTE.dataEnd;
                     ibytes = previousTE.bytes;
                 }
 
                 if (i == numTranscoders - 1) {
-                    opp.p = outPtr.p;
+                    opp = outPtr;
                     os = outStop;
                     obytes = out;
                 } else {
@@ -170,7 +214,8 @@ public final class EConv implements EConvFlags {
                         te.dataStart = te.bufStart;
                         te.dataEnd -= off;
                     }
-                    opp.p = te.dataEnd;
+                    opp = new Ptr(te.dataEnd);
+                    oppIsEnd = true;
                     os = te.bufEnd;
                     obytes = te.bytes;
                 }
@@ -190,6 +235,9 @@ public final class EConv implements EConvFlags {
                 EConvResult res;
                 te.lastResult = res = te.transcoding.convert(ibytes, ipp, is, obytes, opp, os, f);
 
+                if (ippIsStart) previousTE.dataStart = ipp.p;
+                if (oppIsEnd) te.dataEnd = opp.p;
+
                 if (iold != ipp.p || oold != opp.p) try_ = true;
 
                 switch (res) {
@@ -220,7 +268,7 @@ public final class EConv implements EConvFlags {
 
         if (elements[0].lastResult == EConvResult.AfterOutput) elements[0].lastResult = EConvResult.SourceBufferEmpty;
 
-        for (int i = numTranscoders = 1; i >= 0; i--) {
+        for (int i = numTranscoders - 1; 0 <= i; i--) {
             switch (elements[i].lastResult) {
             case InvalidByteSequence:
             case IncompleteInput:
@@ -289,7 +337,7 @@ public final class EConv implements EConvFlags {
                     return convertInternalResult(EConvResult.DestinationBufferFull, null);
                 }
                 len = inBuf.dataEnd - inBuf.dataStart;
-                System.arraycopy(inBuf, inBuf.dataStart, out, outPtr.p, len);
+                System.arraycopy(inBuf.bytes, inBuf.dataStart, out, outPtr.p, len);
                 outPtr.p += len;
                 inBuf.dataStart = inBuf.dataEnd = inBuf.bufStart;
                 if ((flags & AFTER_OUTPUT) != 0) return convertInternalResult(EConvResult.AfterOutput, null);
@@ -328,7 +376,7 @@ public final class EConv implements EConvFlags {
             byte[] data = elem.bytes;
             if (dataStart != dataEnd) {
                 if (outStop - outPtr.p < dataEnd - dataStart) {
-                    len = outStop = outPtr.p;
+                    len = outStop - outPtr.p;
                     System.arraycopy(data, dataStart, out, outPtr.p, len);
                     outPtr.p = outStop;
                     elem.dataStart += len;
@@ -378,17 +426,27 @@ public final class EConv implements EConvFlags {
             lastError.destination = errorTranscoding.transcoder.destination;
             lastError.errorBytes = errorTranscoding.readBuf;
             lastError.errorBytesP = 0;
-            lastError.errorBytesEnd = errorTranscoding.recognizedLength; // ???
+            lastError.errorBytesLength = errorTranscoding.recognizedLength;
             lastError.readAgainLength = errorTranscoding.readAgainLength;
         }
         return res;
     }
 
     /* rb_econv_convert */
-    private EConvResult convert(byte[] in, Ptr inPtr, int inStop, byte[] out, Ptr outPtr, int outStop, int flags) {
+    public EConvResult convert(byte[] in, Ptr inPtr, int inStop, byte[] out, Ptr outPtr, int outStop, int flags) {
         started = true;
 
-        // null check
+        if (in == null || inPtr == null) {
+            in = NULL_STRING;
+            inPtr = Ptr.NULL;
+            inStop = 0;
+        }
+
+        if (out == null || outPtr == null) {
+            out = NULL_STRING;
+            outPtr = Ptr.NULL;
+            outStop = 0;
+        }
 
         resume: while (true) {
             EConvResult ret = convertInternal(in, inPtr, inStop, out, outPtr, outStop, flags);
@@ -422,12 +480,13 @@ public final class EConv implements EConvFlags {
         if (caseInsensitiveEquals(lastError.source, "UTF-32BE".getBytes())) {
             utfBytes = lastError.errorBytes;
             utfP = lastError.errorBytesP;
-            utfLen = lastError.errorBytesEnd - lastError.errorBytesP;
+            utfLen = lastError.errorBytesLength;
         } else {
             Ptr utfLenA = new Ptr();
-            byte[] utfBuf = new byte[1024]; // ??
-            utfBytes = allocateConvertedString(lastError.source, "UTF-32BE".getBytes(), lastError.errorBytes, lastError.errorBytesP, lastError.errorBytesEnd
-                    - lastError.errorBytesP, utfBuf, utfLenA);
+
+            // TODO: better calculation?
+            byte[] utfBuf = new byte[lastError.errorBytesLength * UTF32BEEncoding.INSTANCE.maxLength()];
+            utfBytes = allocateConvertedString(lastError.source, "UTF-32BE".getBytes(), lastError.errorBytes, lastError.errorBytesP, lastError.errorBytesLength, utfBuf, utfLenA);
 
             if (utfBytes == null) return -1;
             utfP = 0;
@@ -442,10 +501,10 @@ public final class EConv implements EConvFlags {
             u += (utfBytes[p] & 0xff) << 24;
             u += (utfBytes[p + 1] & 0xff) << 16;
             u += (utfBytes[p + 2] & 0xff) << 8;
-            u += (utfBytes[p + 3]);
-            byte[] charrefbuf = String.format("&#x%X;", u).getBytes(); // use faster sprintf ??
+            u += (utfBytes[p + 3] & 0xff);
+            byte[] charrefbuf = String.format("&#x%X;", u).getBytes(); // FIXME: use faster sprintf ??
 
-            if (insertOuput(charrefbuf, charrefbuf.length, "US-ASCII".getBytes()) == -1) return -1;
+            if (insertOutput(charrefbuf, 0, charrefbuf.length, "US-ASCII".getBytes()) == -1) return -1;
 
             p += 4;
             utfLen -= 4;
@@ -455,7 +514,7 @@ public final class EConv implements EConvFlags {
     }
 
     /* rb_econv_encoding_to_insert_output */
-    private byte[] encodingToInsertOutput() {
+    public byte[] encodingToInsertOutput() {
         Transcoding transcoding = lastTranscoding;
         if (transcoding == null) return NULL_STRING;
         Transcoder transcoder = transcoding.transcoder;
@@ -488,6 +547,7 @@ public final class EConv implements EConvFlags {
         Ptr sp = new Ptr(strP);
         Ptr dp = new Ptr(dstLen);
         EConvResult res = ec.convert(str, sp, strP + strLen, dstStr, dp, dstBufSize, 0);
+        dstLen = dp.p;
 
         while (res.isDestinationBufferFull()) {
             dstBufSize *= 2;
@@ -509,23 +569,28 @@ public final class EConv implements EConvFlags {
     }
 
     /* rb_econv_insert_output */
-    int insertOuput(byte[] str, int strLen, byte[] strEncoding) {
+    public int insertOutput(byte[] str, int strP, int strLen, byte[] strEncoding) {
         byte[] insertEncoding = encodingToInsertOutput();
-        byte[] insertBuf = new byte[4096];
+        byte[] insertBuf = null;
 
         started = true;
 
         if (strLen == 0) return 0;
 
         final byte[] insertStr;
+        final int insertP;
         final int insertLen;
         if (caseInsensitiveEquals(insertEncoding, strEncoding)) {
             insertStr = str;
+            insertP = 0;
             insertLen = strLen;
         } else {
             Ptr insertLenP = new Ptr();
-            insertStr = allocateConvertedString(strEncoding, insertEncoding, str, 0, strLen, insertBuf, insertLenP);
+            insertBuf = new byte[4096]; // FIXME: wasteful
+            insertStr = allocateConvertedString(strEncoding, insertEncoding, str, strP, strLen, insertBuf, insertLenP);
             insertLen = insertLenP.p;
+            insertP = insertStr == str ? strP : 0;
+            if (insertStr == null) return -1;
         }
 
         int need = insertLen;
@@ -556,29 +621,34 @@ public final class EConv implements EConvFlags {
         if (buf == null) {
             buf = new Buffer();
             buf.allocate(need);
-        } else if (need > (buf.bufEnd - buf.dataEnd)) {
+        } else if (buf.bytes == null) {
+            buf.bytes = new byte[need];
+            buf.dataStart = 0;
+            buf.dataEnd = 0;
+            buf.bufEnd = need;
+        } else if ((buf.bufEnd - buf.dataEnd) < need) {
+            // try to compact buffer by moving data portion back to bufStart
             System.arraycopy(buf.bytes, buf.dataStart, buf.bytes, buf.bufStart, buf.dataEnd - buf.dataStart);
-            buf.dataEnd = buf.dataStart + (buf.dataEnd - buf.dataStart);
+            buf.dataEnd = buf.bufStart + (buf.dataEnd - buf.dataStart);
             buf.dataStart = buf.bufStart;
 
-            if (need > (buf.bufEnd - buf.dataEnd)) {
+            if ((buf.bufEnd - buf.dataEnd) < need) {
+                // still not enough room; use a separate buffer
                 int s = (buf.dataEnd - buf.bufStart) + need;
-                if (need > s) return -1;
-                byte[] tmp = new byte[s];
-                System.arraycopy(buf.bytes, buf.bufStart, tmp, 0, s); // ??
-                buf.bytes = tmp;
-                buf.dataStart = 0;
-                buf.dataEnd = buf.dataEnd - buf.bufStart;
-                buf.bufStart = 0;
-                buf.bufEnd = 0;
+                if (s < need) return -1;
+                Buffer buf2 = buf = new Buffer();
+                buf2.allocate(s);
+                System.arraycopy(buf.bytes, buf.bufStart, buf2.bytes, 0, s); // ??
+                buf2.dataStart = 0;
+                buf2.dataEnd = buf.dataEnd - buf.bufStart;
             }
         }
 
-        System.arraycopy(insertStr, 0, buf, buf.dataEnd, insertLen);
+        System.arraycopy(insertStr, insertP, buf.bytes, buf.dataEnd, insertLen);
         buf.dataEnd += insertLen;
         if (transcoding != null && transcoding.transcoder.compatibility.isEncoder()) {
-            System.arraycopy(transcoding.readBuf, transcoding.recognizedLength, buf, buf.dataEnd, transcoding.readAgainLength);
-            buf.dataEnd = transcoding.readAgainLength;
+            System.arraycopy(transcoding.readBuf, transcoding.recognizedLength, buf.bytes, buf.dataEnd, transcoding.readAgainLength);
+            buf.dataEnd += transcoding.readAgainLength;
             transcoding.readAgainLength = 0;
         }
 
@@ -586,27 +656,27 @@ public final class EConv implements EConvFlags {
     }
 
     /* rb_econv_close */
-    void close() {
+    public void close() {
         for (int i = 0; i < numTranscoders; i++) {
             elements[i].transcoding.close();
         }
     }
 
     /* rb_econv_putbackable */
-    int putbackable() {
+    public int putbackable() {
         return numTranscoders == 0 ? 0 : elements[0].transcoding.readAgainLength;
     }
 
     /* rb_econv_putback */
-    void putback(byte[] bytes, int p, int len) {
-        if (numTranscoders == 0 || len == 0) return;
+    public void putback(byte[] bytes, int p, int n) {
+        if (numTranscoders == 0 || n == 0) return;
         Transcoding transcoding = elements[0].transcoding;
-        System.arraycopy(transcoding.readBuf, transcoding.recognizedLength + transcoding.readAgainLength, bytes, p, len);
-        transcoding.readAgainLength -= len;
+        System.arraycopy(transcoding.readBuf, transcoding.recognizedLength + transcoding.readAgainLength - n, bytes, p, n);
+        transcoding.readAgainLength -= n;
     }
 
     /* rb_econv_add_converter */
-    boolean addConverter(byte[] source, byte[] destination, int n) {
+    public boolean addConverter(byte[] source, byte[] destination, int n) {
         if (started) return false;
         TranscoderDB.Entry entry = TranscoderDB.getEntry(source, destination);
         if (entry == null) return false;
@@ -621,7 +691,8 @@ public final class EConv implements EConvFlags {
         return addConverter(NULL_STRING, decorator, n);
     }
 
-    /* rb_econv_decorate_at_last */
+    /* rb_econv_decorate_at_first */
+    // NOTE: Unused in MRI proper; for C exts, presumably
     boolean decorateAtFirst(byte[] decorator) {
         if (numTranscoders == 0) return decorateAt(decorator, 0);
         Transcoder transcoder = elements[0].transcoding.transcoder;
@@ -638,7 +709,7 @@ public final class EConv implements EConvFlags {
         if (numTranscoders == 0) return decorateAt(decorator, 0);
         Transcoder transcoder = elements[numTranscoders - 1].transcoding.transcoder;
 
-        if (!decorator(transcoder.source, transcoder.destination) && transcoder.compatibility.isDecoder()) {
+        if (!decorator(transcoder.source, transcoder.destination) && transcoder.compatibility.isEncoder()) {
             return decorateAt(decorator, numTranscoders - 1);
         }
 
@@ -646,7 +717,7 @@ public final class EConv implements EConvFlags {
     }
 
     /* rb_econv_binmode */
-    private void binmode() {
+    public void binmode() {
         Transcoder[] transcoders = new Transcoder[3];
         int n = 0;
         if ((flags & UNIVERSAL_NEWLINE_DECORATOR) != 0) {
@@ -688,7 +759,7 @@ public final class EConv implements EConvFlags {
     /* more_output_buffer */
 
     /* make_replacement */
-    private int makeReplacement() {
+    public int makeReplacement() {
         if (replacementString != null) return 0;
 
         byte[] insEnc = encodingToInsertOutput();
@@ -725,7 +796,7 @@ public final class EConv implements EConvFlags {
     }
 
     /* rb_econv_set_replacement */
-    int setReplacement(byte[] str, int p, int len, byte[] encname) {
+    public int setReplacement(byte[] str, int p, int len, byte[] encname) {
         byte[] encname2 = encodingToInsertOutput();
 
         final byte[] str2;
@@ -740,7 +811,7 @@ public final class EConv implements EConvFlags {
         } else {
             Ptr len2p = new Ptr();
             str2 = allocateConvertedString(encname, encname2, str, p, len, null, len2p);
-            if (str == null) return -1;
+            if (str2 == null) return -1;
             len2 = len2p.p;
         }
 
@@ -753,7 +824,7 @@ public final class EConv implements EConvFlags {
     /* output_replacement_character */
     int outputReplacementCharacter() {
         if (makeReplacement() == -1) return -1;
-        if (insertOuput(replacementString, replacementLength, replacementEncoding) == -1) return -1;
+        if (insertOutput(replacementString, 0, replacementLength, replacementEncoding) == -1) return -1;
         return 0;
     }
 
@@ -773,4 +844,44 @@ public final class EConv implements EConvFlags {
 
         return s;
     }
+
+    // econv_equal
+    @Override
+    public boolean equals(Object other) {
+        if (!(other instanceof EConv)) return false;
+
+        EConv ec1 = this;
+        EConv ec2 = (EConv)other;
+        int i;
+
+        if (ec2 == null) return false;
+        if (ec1.source != ec2.source &&
+                !Arrays.equals(ec1.source, ec2.source))
+            return false;
+        if (ec1.destination != ec2.destination &&
+                !Arrays.equals(ec1.destination, ec2.destination))
+            return false;
+        if (ec1.flags != ec2.flags) return false;
+        if (ec1.replacementEncoding != ec2.replacementEncoding &&
+                !Arrays.equals(ec1.replacementEncoding, ec2.replacementEncoding))
+            return false;
+        if (ec1.replacementLength != ec2.replacementLength) return false;
+        if (ec1.replacementString != ec2.replacementString &&
+                !memcmp(ec1.replacementString, ec2.replacementString, ec2.replacementLength))
+            return false;
+
+        if (ec1.numTranscoders != ec2.numTranscoders) return false;
+        for (i = 0; i < ec1.numTranscoders; i++) {
+            if (ec1.elements[i].transcoding.transcoder != ec2.elements[i].transcoding.transcoder)
+                return false;
+        }
+        return true;
+    }
+
+    private static boolean memcmp(byte[] a, byte[] b, int len) {
+        for (int i = 0; i < len; i++) {
+            if (a[i] != b[i]) return false;
+        }
+        return true;
+    }
 }
\ No newline at end of file
