@@ -80,9 +80,11 @@ final class TranscoderList {
         {"EUC-JP", "UTF-8", null /*From_EUC_JP*/},
         {"eucJP-ms", "UTF-8", null /*From_eucJP_ms*/},
         {"CP51932", "UTF-8", null /*From_CP51932*/},
+        {"EUC-JIS-2004", "UTF-8", null /*From_EUC_JIS_2004*/},
         {"UTF-8", "EUC-JP", null /*To_EUC_JP*/},
         {"UTF-8", "eucJP-ms", null /*To_eucJP_ms*/},
         {"UTF-8", "CP51932", null /*To_CP51932*/},
+        {"UTF-8", "EUC-JIS-2004", null /*To_EUC_JIS_2004*/},
         {"Shift_JIS", "UTF-8", null /*From_Shift_JIS*/},
         {"Windows-31J", "UTF-8", null /*From_Windows_31J*/},
         {"UTF-8", "Shift_JIS", null /*To_Shift_JIS*/},
@@ -123,6 +125,8 @@ final class TranscoderList {
         {"UTF-8", "ISO-8859-14", null /*To_ISO_8859_14*/},
         {"ISO-8859-15", "UTF-8", null /*From_ISO_8859_15*/},
         {"UTF-8", "ISO-8859-15", null /*To_ISO_8859_15*/},
+        {"ISO-8859-16", "UTF-8", null /*From_ISO_8859_16*/},
+        {"UTF-8", "ISO-8859-16", null /*To_ISO_8859_16*/},
         {"WINDOWS-874", "UTF-8", null /*From_WINDOWS_874*/},
         {"UTF-8", "WINDOWS-874", null /*To_WINDOWS_874*/},
         {"WINDOWS-1250", "UTF-8", null /*From_WINDOWS_1250*/},
@@ -208,7 +212,10 @@ final class TranscoderList {
         {"UTF-16", "UTF-8", "From_UTF_16"},
         {"UTF-32", "UTF-8", "From_UTF_32"},
         {"UTF-8", "UTF-16", "To_UTF_16"},
-        {"UTF-8", "UTF-32", "To_UTF_32"}
+        {"UTF-8", "UTF-32", "To_UTF_32"},
+        {"", "universal_newline", "Universal_newline"},
+        {"", "crlf_newline", null /*Crlf_newline*/},
+        {"", "cr_newline", null /*Cr_newline*/}
     };
 
     static final GenericTranscoderEntry[] GENERIC_LIST = new GenericTranscoderEntry[] {
@@ -269,17 +276,19 @@ final class TranscoderList {
         new GenericTranscoderEntry("EUC-JP", "UTF-8", 54488, "JapaneseEuc", 1, 3, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("eucJP-ms", "UTF-8", 64480, "JapaneseEuc", 1, 3, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("CP51932", "UTF-8", 66380, "JapaneseEuc", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "EUC-JP", 125780, "JapaneseEuc", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "eucJP-ms", 144752, "JapaneseEuc", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "CP51932", 179440, "JapaneseEuc", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("EUC-JIS-2004", "UTF-8", 86460, "JapaneseEuc", 1, 3, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "EUC-JP", 145860, "JapaneseEuc", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "eucJP-ms", 164832, "JapaneseEuc", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "CP51932", 199520, "JapaneseEuc", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "EUC-JIS-2004", 257272, "JapaneseEuc", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("Shift_JIS", "UTF-8", 28448, "JapaneseSjis", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("Windows-31J", "UTF-8", 40648, "JapaneseSjis", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("UTF-8", "Shift_JIS", 75704, "JapaneseSjis", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("UTF-8", "Windows-31J", 104500, "JapaneseSjis", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "EUC-KR", 44104, "Korean", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("EUC-KR", "UTF-8", 78484, "Korean", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "CP949", 126392, "Korean", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("CP949", "UTF-8", 196860, "Korean", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "EUC-KR", 44112, "Korean", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("EUC-KR", "UTF-8", 78500, "Korean", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "CP949", 126176, "Korean", 1, 4, 2, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("CP949", "UTF-8", 196644, "Korean", 1, 2, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("US-ASCII", "UTF-8", 8, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("UTF-8", "US-ASCII", 132, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("ASCII-8BIT", "UTF-8", 148, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
@@ -312,80 +321,82 @@ final class TranscoderList {
         new GenericTranscoderEntry("UTF-8", "ISO-8859-14", 14452, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("ISO-8859-15", "UTF-8", 14976, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
         new GenericTranscoderEntry("UTF-8", "ISO-8859-15", 15360, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-874", "UTF-8", 15764, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-874", 15928, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1250", "UTF-8", 16436, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1250", 16992, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1251", "UTF-8", 17516, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1251", 18152, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1252", "UTF-8", 18660, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1252", 18960, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1253", "UTF-8", 19420, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1253", 19932, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1254", "UTF-8", 20432, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1254", 20548, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1255", "UTF-8", 20984, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1255", 21756, "SingleByte", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1256", "UTF-8", 22280, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1256", 23024, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("WINDOWS-1257", "UTF-8", 23504, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "WINDOWS-1257", 23724, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM437", "UTF-8", 24248, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM437", 25084, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM775", "UTF-8", 25608, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM775", 26252, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM737", "UTF-8", 26776, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM737", 27288, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM852", "UTF-8", 27812, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM852", 28428, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM855", "UTF-8", 28952, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM855", 29504, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM857", "UTF-8", 30020, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM857", 30532, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM860", "UTF-8", 31056, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM860", 31444, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM861", "UTF-8", 31968, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM861", 32280, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM862", "UTF-8", 32804, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM862", 33048, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM863", "UTF-8", 33572, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM863", 33952, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM865", "UTF-8", 34476, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM865", 34788, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM866", "UTF-8", 35312, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM866", 35768, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("IBM869", "UTF-8", 36260, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "IBM869", 36776, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACCROATIAN", "UTF-8", 37300, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACCROATIAN", 38132, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACCYRILLIC", "UTF-8", 38656, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACCYRILLIC", 39360, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACGREEK", "UTF-8", 39884, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACGREEK", 40584, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACICELAND", "UTF-8", 41108, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACICELAND", 41824, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACROMAN", "UTF-8", 42348, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACROMAN", 42832, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACROMANIA", "UTF-8", 43356, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACROMANIA", 43732, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACTURKISH", "UTF-8", 44252, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACTURKISH", 44608, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("MACUKRAINE", "UTF-8", 45132, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "MACUKRAINE", 45356, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("KOI8-U", "UTF-8", 45880, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "KOI8-U", 46664, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("KOI8-R", "UTF-8", 47188, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "KOI8-R", 47720, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("TIS-620", "UTF-8", 48084, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "TIS-620", 48128, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("CP850", "UTF-8", 48652, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "CP850", 49200, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("CP852", "UTF-8", 27812, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "CP852", 28428, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("CP855", "UTF-8", 28952, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "CP855", 29504, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF-8", "UTF8-MAC", 4804, "Utf8Mac", 1, 4, 9, AsciiCompatibility.CONVERTER, 0),
-        new GenericTranscoderEntry("UTF8-MAC", "UTF-8", 4928, "Utf8Mac", 1, 4, 10, AsciiCompatibility.ENCODER, 0),
+        new GenericTranscoderEntry("ISO-8859-16", "UTF-8", 15884, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "ISO-8859-16", 16580, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-874", "UTF-8", 16984, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-874", 17148, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1250", "UTF-8", 17656, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1250", 18212, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1251", "UTF-8", 18736, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1251", 19372, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1252", "UTF-8", 19880, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1252", 20180, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1253", "UTF-8", 20640, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1253", 21152, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1254", "UTF-8", 21652, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1254", 21768, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1255", "UTF-8", 22204, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1255", 22976, "SingleByte", 1, 4, 3, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1256", "UTF-8", 23500, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1256", 24244, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("WINDOWS-1257", "UTF-8", 24724, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "WINDOWS-1257", 24944, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM437", "UTF-8", 25468, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM437", 26304, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM775", "UTF-8", 26828, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM775", 27472, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM737", "UTF-8", 27996, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM737", 28508, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM852", "UTF-8", 29032, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM852", 29648, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM855", "UTF-8", 30172, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM855", 30724, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM857", "UTF-8", 31240, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM857", 31752, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM860", "UTF-8", 32276, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM860", 32664, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM861", "UTF-8", 33188, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM861", 33500, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM862", "UTF-8", 34024, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM862", 34268, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM863", "UTF-8", 34792, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM863", 35172, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM865", "UTF-8", 35696, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM865", 36008, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM866", "UTF-8", 36532, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM866", 36988, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("IBM869", "UTF-8", 37480, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "IBM869", 37996, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACCROATIAN", "UTF-8", 38520, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACCROATIAN", 39352, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACCYRILLIC", "UTF-8", 39876, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACCYRILLIC", 40580, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACGREEK", "UTF-8", 41104, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACGREEK", 41804, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACICELAND", "UTF-8", 42328, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACICELAND", 43044, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACROMAN", "UTF-8", 43568, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACROMAN", 44052, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACROMANIA", "UTF-8", 44576, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACROMANIA", 44952, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACTURKISH", "UTF-8", 45472, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACTURKISH", 45828, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("MACUKRAINE", "UTF-8", 46352, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "MACUKRAINE", 46576, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("KOI8-U", "UTF-8", 47100, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "KOI8-U", 47884, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("KOI8-R", "UTF-8", 48408, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "KOI8-R", 48940, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("TIS-620", "UTF-8", 49304, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "TIS-620", 49348, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("CP850", "UTF-8", 49872, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "CP850", 50420, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("CP852", "UTF-8", 29032, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "CP852", 29648, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("CP855", "UTF-8", 30172, "SingleByte", 1, 1, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "CP855", 30724, "SingleByte", 1, 4, 1, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF-8", "UTF8-MAC", 52420, "Utf8Mac", 1, 4, 9, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("UTF8-MAC", "UTF-8", 52544, "Utf8Mac", 1, 4, 10, AsciiCompatibility.ENCODER, 0),
         new GenericTranscoderEntry("UTF-16BE", "UTF-8", 188, "Utf1632", 2, 4, 4, AsciiCompatibility.DECODER, 0),
         new GenericTranscoderEntry("UTF-8", "UTF-16BE", 416, "Utf1632", 1, 4, 4, AsciiCompatibility.ENCODER, 0),
         new GenericTranscoderEntry("UTF-16LE", "UTF-8", 52, "Utf1632", 2, 4, 4, AsciiCompatibility.DECODER, 0),
@@ -397,6 +408,9 @@ final class TranscoderList {
         new GenericTranscoderEntry("UTF-16", "UTF-8", 276, "Utf1632", 2, 4, 4, AsciiCompatibility.DECODER, 1),
         new GenericTranscoderEntry("UTF-32", "UTF-8", 300, "Utf1632", 4, 4, 4, AsciiCompatibility.DECODER, 1),
         new GenericTranscoderEntry("UTF-8", "UTF-16", 416, "Utf1632", 1, 4, 4, AsciiCompatibility.ENCODER, 1),
-        new GenericTranscoderEntry("UTF-8", "UTF-32", 416, "Utf1632", 1, 4, 4, AsciiCompatibility.ENCODER, 1)
+        new GenericTranscoderEntry("UTF-8", "UTF-32", 416, "Utf1632", 1, 4, 4, AsciiCompatibility.ENCODER, 1),
+        new GenericTranscoderEntry("", "universal_newline", 4, "Newline", 1, 1, 2, AsciiCompatibility.CONVERTER, 2),
+        new GenericTranscoderEntry("", "crlf_newline", 20, "Newline", 1, 1, 2, AsciiCompatibility.CONVERTER, 0),
+        new GenericTranscoderEntry("", "cr_newline", 36, "Newline", 1, 1, 1, AsciiCompatibility.CONVERTER, 0)
     };
 }
\ No newline at end of file
